@@ -46,6 +46,12 @@
 #include "logging.h"
 #include "support.h"
 #include "setup.h"
+
+#ifdef __MINGW32__
+#ifndef __MINGW64__
+#pragma push_macro("__inline__")
+#endif
+#endif
 #include "src/libs/decoders/audio_convert.c"
 #include "src/libs/decoders/SDL_sound.c"
 #include "src/libs/decoders/vorbis.c"
@@ -54,27 +60,29 @@
 #include "src/libs/decoders/wav.c"
 #include "src/libs/decoders/mp3_seek_table.cpp"
 #include "src/libs/decoders/mp3.cpp"
+#include "src/libs/decoders/dr_flac.h"
+#ifdef __MINGW32__
+#ifndef __MINGW64__
+#pragma pop_macro("__inline__")
+#endif
+#endif
 #include "src/libs/libchdr/chd.h"
 #include "src/libs/libchdr/libchdr_chd.c"
 #include "src/libs/libchdr/libchdr_cdrom.c"
 #include "src/libs/libchdr/libchdr_flac.c"
 #include "src/libs/libchdr/libchdr_huffman.c"
 #include "src/libs/libchdr/libchdr_bitstream.c"
-#include "src/libs/libchdr/FLAC/stream_decoder.c"
-#include "src/libs/libchdr/FLAC/bitreader.c"
-#include "src/libs/libchdr/FLAC/format.c"
-#include "src/libs/libchdr/FLAC/cpu.c"
-#include "src/libs/libchdr/FLAC/crc.c"
-#include "src/libs/libchdr/FLAC/fixed.c"
-#include "src/libs/libchdr/FLAC/lpc.c"
-#include "src/libs/libchdr/FLAC/md5.c"
-#include "src/libs/libchdr/FLAC/memory.c"
-#if defined(WIN32) && !defined(HX_DOS) && !(defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR))
-#include "src/libs/libchdr/FLAC/windows_unicode_filenames.c"
-#endif
 #include "src/libs/libchdr/lzma/LzmaDec.c"
 #include "src/libs/libchdr/lzma/LzmaEnc.c"
 #include "src/libs/libchdr/lzma/LzFind.c"
+#include "src/libs/libchdr/zstd/common/entropy_common.c"
+#include "src/libs/libchdr/zstd/common/error_private.c"
+#include "src/libs/libchdr/zstd/common/fse_decompress.c"
+#include "src/libs/libchdr/zstd/common/zstd_common.c"
+#include "src/libs/libchdr/zstd/decompress/huf_decompress.c"
+#include "src/libs/libchdr/zstd/decompress/zstd_decompress_block.c"
+#include "src/libs/libchdr/zstd/decompress/zstd_decompress.c"
+#include "src/libs/libchdr/zstd/decompress/zstd_ddict.c"
 
 using namespace std;
 
@@ -580,8 +588,8 @@ bool CDROM_Interface_Image::GetAudioSub(unsigned char& attr, unsigned char& trac
 	track = (unsigned char)cur_track;
 	attr = tracks[track - 1].attr;
 	index = 1;
-	FRAMES_TO_MSF(player.currFrame + 150, &absPos.min, &absPos.sec, &absPos.fr);
-	FRAMES_TO_MSF(player.currFrame - tracks[track - 1].start + 150, &relPos.min, &relPos.sec, &relPos.fr);
+	FRAMES_TO_MSF(player.currFrame, &absPos.min, &absPos.sec, &absPos.fr);
+	FRAMES_TO_MSF(player.currFrame - tracks[track - 1].start, &relPos.min, &relPos.sec, &relPos.fr);
 	if(IS_PC98_ARCH && player.playbackRemaining == 0 && !strcmp(RunningProgram, "ITP")) {
 		// POLICENAUTS
 		// It freeze at the end of the Konami logo or opening.
@@ -596,13 +604,13 @@ bool CDROM_Interface_Image::GetAudioSub(unsigned char& attr, unsigned char& trac
 
 	LOG_MSG("%s CDROM: GetAudioSub absolute offset (%d), MSF=%d:%d:%d",
       get_time(),
-	  player.currFrame + 150,
+	  player.currFrame,
 	  absPos.min,
 	  absPos.sec,
 	  absPos.fr);
 	LOG_MSG("%s CDROM: GetAudioSub relative offset (%d), MSF=%d:%d:%d",
       get_time(),
-	  player.currFrame - tracks[track - 1].start + 150,
+	  player.currFrame - tracks[track - 1].start,
 	  relPos.min,
 	  relPos.sec,
 	  relPos.fr);
@@ -1305,7 +1313,7 @@ bool CDROM_Interface_Image::LoadCueSheet(char *cuefile)
 	{
 		char *s = strrchr(cuefile,'.');
 		if (!s) return false;
-		if (!strcasecmp(s,".ccd") || !strcasecmp(s, ".chd") || !strcasecmp(s, ".iso") || !strcasecmp(s, ".img")
+		if (!strcasecmp(s,".ccd") || !strcasecmp(s, ".chd") || !strcasecmp(s, ".iso") || !strcasecmp(s, ".img") || !strcasecmp(s, ".gog")
             || !strcasecmp(s, ".mds") || !strcasecmp(s, ".mdf") || !strcasecmp(s, ".bin")) return false;
 	}
 
