@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -32,17 +32,18 @@
 #include <oleauto.h>
 
 #ifndef SDL_DISABLE_WINDOWS_IME
-static Uint32 end_ticks = 0;  // added for DOSBox-X
-static SDL_bool ime_incompos; // added for DOSBox-X
 static void IME_Init(SDL_VideoData *videodata, HWND hwnd);
 static void IME_Enable(SDL_VideoData *videodata, HWND hwnd);
 static void IME_Disable(SDL_VideoData *videodata, HWND hwnd);
 static void IME_Quit(SDL_VideoData *videodata);
 static SDL_bool IME_IsTextInputShown(SDL_VideoData *videodata);
+static Uint32 end_ticks = 0;  // added for DOSBox-X
+static SDL_bool ime_incompos; // added for DOSBox-X
 #endif /* !SDL_DISABLE_WINDOWS_IME */
 
 #if 1 // Added for DOSBox-X
-SDL_bool SDL_IM_Composition(int more) {
+SDL_bool SDL_IM_Composition(int more)
+{
     (void)more;
 #ifndef SDL_DISABLE_WINDOWS_IME
 #define IME_END_CR_WAIT 50
@@ -180,7 +181,7 @@ void WIN_QuitKeyboard(_THIS)
 #endif /* !SDL_DISABLE_WINDOWS_IME */
 }
 
-void WIN_ResetDeadKeys()
+void WIN_ResetDeadKeys(void)
 {
     /*
     if a deadkey has been typed, but not the next character (which the deadkey might modify),
@@ -266,6 +267,22 @@ void WIN_SetTextInputRect(_THIS, const SDL_Rect *rect)
         /* //reverted for DOSBox-X
         COMPOSITIONFORM cof;
         CANDIDATEFORM caf;
+
+        cof.dwStyle = CFS_RECT;
+        cof.ptCurrentPos.x = videodata->ime_rect.x;
+        cof.ptCurrentPos.y = videodata->ime_rect.y;
+        cof.rcArea.left = videodata->ime_rect.x;
+        cof.rcArea.right = (LONG)videodata->ime_rect.x + videodata->ime_rect.w;
+        cof.rcArea.top = videodata->ime_rect.y;
+        cof.rcArea.bottom = (LONG)videodata->ime_rect.y + videodata->ime_rect.h;
+        ImmSetCompositionWindow(himc, &cof);
+
+        caf.dwIndex = 0;
+        caf.dwStyle = CFS_EXCLUDE;
+        caf.ptCurrentPos.x = videodata->ime_rect.x;
+        caf.ptCurrentPos.y = videodata->ime_rect.y;
+        caf.rcArea.left = videodata->ime_rect.x;
+        caf.rcArea.right = (LONG)videodata->ime_rect.x + videodata->ime_rect.w;
         caf.rcArea.top = videodata->ime_rect.y;
         caf.rcArea.bottom = (LONG)videodata->ime_rect.y + videodata->ime_rect.h;
         ImmSetCandidateWindow(himc, &caf);
@@ -275,7 +292,6 @@ void WIN_SetTextInputRect(_THIS, const SDL_Rect *rect)
         cf.ptCurrentPos.y = videodata->ime_rect.y;
         cf.dwStyle = CFS_FORCE_POSITION;
         ImmSetCompositionWindow(himc, &cf);
-
         ImmReleaseContext(videodata->ime_hwnd_current, himc);
     }
 #endif /* !SDL_DISABLE_WINDOWS_IME */
@@ -372,7 +388,7 @@ static void UILess_ReleaseSinks(SDL_VideoData *videodata);
 static void UILess_EnableUIUpdates(SDL_VideoData *videodata);
 static void UILess_DisableUIUpdates(SDL_VideoData *videodata);
 
-static SDL_bool WIN_ShouldShowNativeUI()
+static SDL_bool WIN_ShouldShowNativeUI(void)
 {
     return SDL_GetHintBoolean(SDL_HINT_IME_SHOW_UI, SDL_FALSE);
 }
@@ -421,7 +437,7 @@ static void IME_Init(SDL_VideoData *videodata, HWND hwnd)
     IME_UpdateInputLocale(videodata);
     IME_SetupAPI(videodata);
     // Disabled because the candidate window will not be displayed. (for DOSBox-X)
-    /*
+    /**
     if (WIN_ShouldShowNativeUI()) {
         videodata->ime_uiless = SDL_FALSE;
     } else {
@@ -1028,11 +1044,10 @@ SDL_bool IME_HandleMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM *lParam, S
         break;
 #if 1 // added for DOSBox-X
     case WM_IME_CHAR:
-        if(wParam == 0x20) {
+        if (wParam == 0x20) {
             // enable IME input space
             PostMessage(hwnd, WM_KEYDOWN, 0x20, 0x390001);
-        }
-        else if(wParam == 0x3000) {
+        } else if (wParam == 0x3000) {
             // input Zenkaku space
             videodata->ime_composition[0] = 0x3000;
             videodata->ime_composition[1] = 0;
@@ -1048,15 +1063,15 @@ SDL_bool IME_HandleMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM *lParam, S
         if (videodata->ime_uiless) {
             *lParam = 0;
         }
-        break;
         */
+        break;
     case WM_IME_STARTCOMPOSITION:
         videodata->ime_suppress_endcomposition_event = SDL_FALSE;
-        ime_incompos = 1;  /* added for DOSBox-X */
-        //trap = SDL_TRUE; /* disabled for DOSBox-X */
+        ime_incompos = 1; /* added for DOSBox-X */
+        // trap = SDL_TRUE; /* disabled for DOSBox-X */
         break;
     case WM_IME_COMPOSITION:
-        //trap = SDL_TRUE; /* disabled for DOSBox-X */
+        // trap = SDL_TRUE; /* disabled for DOSBox-X */
         himc = ImmGetContext(hwnd);
         if (*lParam & GCS_RESULTSTR) {
             videodata->ime_suppress_endcomposition_event = SDL_TRUE;
